@@ -39,6 +39,7 @@ SEXP psll_kill(SEXP p);
 SEXP psll_num_fds(SEXP p);
 SEXP psll_open_files(SEXP p);
 SEXP psll_interrupt(SEXP p, SEXP ctrlc, SEXP interrupt_path);
+SEXP psll_connections(SEXP p);
 
 /* System API */
 
@@ -52,6 +53,7 @@ SEXP ps__users();
 SEXP ps__init(SEXP psenv, SEXP constenv);
 SEXP ps__kill_if_env(SEXP marker, SEXP after, SEXP pid, SEXP sig);
 SEXP ps__find_if_env(SEXP marker, SEXP after, SEXP pid);
+SEXP ps__inet_ntop(SEXP raw, SEXP fam);
 
 SEXP psp__zombie();
 SEXP psp__waitpid(SEXP pid);
